// K型熱電対温度センサモジュールキット(SPI接続)MAX31855使用(3.3V版)サンプルスケッチ
// switch-science 2012.2.8

#include "SPI.h"

/*
// Arduinoボードの電源電圧が3.3Vの場合、
// 以下のマクロを有効にしてモジュールを直挿しできます。
#define VCC  8
#define GND  9
*/
#define SLAVE 10

byte enabled = 255;

void SPI_disable() {
  if(enabled != 255) {
    digitalWrite(enabled,HIGH);
    enabled = 255;
  }
}
void SPI_enable(byte slaveselecter) {
  SPI_disable();
  digitalWrite(slaveselecter,LOW);
  enabled = slaveselecter;
}

byte SPI_read() {
  return SPI.transfer(0x00);
}

void setup() {
#ifdef GND
  pinMode(GND, OUTPUT);
  digitalWrite(GND, LOW); 
#endif
#ifdef VCC
  pinMode(VCC, OUTPUT);
  digitalWrite(VCC, HIGH);
#endif
  pinMode(SLAVE,OUTPUT);
  digitalWrite(SLAVE,HIGH);

  Serial.begin(9600);
  SPI.begin();
  SPI.setBitOrder(MSBFIRST);
  SPI.setClockDivider(SPI_CLOCK_DIV4);
  SPI.setDataMode(SPI_MODE0);
}

void loop() {
  unsigned int thermocouple; // 14-Bit Thermocouple Temperature Data + 2-Bit
  unsigned int internal; // 12-Bit Internal Temperature Data + 4-Bit
  float disp; // display value 

  delay(500);
  SPI_enable(SLAVE);
  thermocouple = (unsigned int)SPI_read() << 8;
  thermocouple |= (unsigned int)SPI_read() ;
  internal = (unsigned int)SPI_read() << 8;
  internal |= (unsigned int)SPI_read();
  SPI_disable();

  if((thermocouple & 0x0001) != 0) {
    Serial.print("ERROR: ");
    if ((internal & 0x0004) !=0) {
      Serial.print("Short to Vcc, ");
    }
    if ((internal & 0x0002) !=0) {
      Serial.print("Short to GND, ");
    }
    if ((internal & 0x0001) !=0) {
      Serial.print("Open Circuit, ");
    }    
    Serial.println("");
  } else {
    if((thermocouple & 0x8000) == 0){ // 0℃以上
      disp = (thermocouple >> 2) * 0.25;
    } else {                          // 0℃未満
      disp = (0x3fff - (thermocouple >> 2) + 1)  * -0.25;
    }
    Serial.print(thermocouple,HEX);
    Serial.print(" : ");
    Serial.print(disp);

    Serial.print(" // ");
    
    if((internal & 0x8000) == 0){ // 0℃以上
      disp = (internal >> 4) * 0.0625;
    } else {                          // 0℃未満
      disp = (((0xffff - internal) >> 4) + 1)  * -0.0625;
    }
    Serial.print(internal,HEX);
    Serial.print(" : ");
    Serial.print(disp);
    
    Serial.println("");    
  }
}

